@@ -23,6 +23,7 @@
 #include <vector>
 
 #include "lib/extras/exif.h"
+#include "lib/jxl/base/common.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/sanitizers.h"
 #if JPEGXL_ENABLE_SJPEG
@@ -50,23 +51,21 @@ enum class JpegEncoder {
   kSJpeg,
 };
 
-#define ARRAY_SIZE(X) (sizeof(X) / sizeof((X)[0]))
-
 // Popular jpeg scan scripts
 // The fields of the individual scans are:
 // comps_in_scan, component_index[], Ss, Se, Ah, Al
-constexpr jpeg_scan_info kScanScript1[] = {
+constexpr auto kScanScript1 = to_array<jpeg_scan_info>({
     {1, {0}, 0, 0, 0, 0},   //
     {1, {1}, 0, 0, 0, 0},   //
     {1, {2}, 0, 0, 0, 0},   //
     {1, {0}, 1, 8, 0, 0},   //
     {1, {0}, 9, 63, 0, 0},  //
     {1, {1}, 1, 63, 0, 0},  //
-    {1, {2}, 1, 63, 0, 0},  //
-};
-constexpr size_t kNumScans1 = ARRAY_SIZE(kScanScript1);
+    {1, {2}, 1, 63, 0, 0}   //
+});
+constexpr size_t kNumScans1 = kScanScript1.size();
 
-constexpr jpeg_scan_info kScanScript2[] = {
+constexpr auto kScanScript2 = to_array<jpeg_scan_info>({
     {1, {0}, 0, 0, 0, 0},   //
     {1, {1}, 0, 0, 0, 0},   //
     {1, {2}, 0, 0, 0, 0},   //
@@ -74,11 +73,11 @@ constexpr jpeg_scan_info kScanScript2[] = {
     {1, {0}, 3, 63, 0, 1},  //
     {1, {0}, 1, 63, 1, 0},  //
     {1, {1}, 1, 63, 0, 0},  //
-    {1, {2}, 1, 63, 0, 0},  //
-};
-constexpr size_t kNumScans2 = ARRAY_SIZE(kScanScript2);
+    {1, {2}, 1, 63, 0, 0}   //
+});
+constexpr size_t kNumScans2 = kScanScript2.size();
 
-constexpr jpeg_scan_info kScanScript3[] = {
+constexpr auto kScanScript3 = to_array<jpeg_scan_info>({
     {1, {0}, 0, 0, 0, 0},   //
     {1, {1}, 0, 0, 0, 0},   //
     {1, {2}, 0, 0, 0, 0},   //
@@ -86,11 +85,11 @@ constexpr jpeg_scan_info kScanScript3[] = {
     {1, {0}, 1, 63, 2, 1},  //
     {1, {0}, 1, 63, 1, 0},  //
     {1, {1}, 1, 63, 0, 0},  //
-    {1, {2}, 1, 63, 0, 0},  //
-};
-constexpr size_t kNumScans3 = ARRAY_SIZE(kScanScript3);
+    {1, {2}, 1, 63, 0, 0}   //
+});
+constexpr size_t kNumScans3 = kScanScript3.size();
 
-constexpr jpeg_scan_info kScanScript4[] = {
+constexpr auto kScanScript4 = to_array<jpeg_scan_info>({
     {3, {0, 1, 2}, 0, 0, 0, 1},  //
     {1, {0}, 1, 5, 0, 2},        //
     {1, {2}, 1, 63, 0, 1},       //
@@ -100,11 +99,11 @@ constexpr jpeg_scan_info kScanScript4[] = {
     {3, {0, 1, 2}, 0, 0, 1, 0},  //
     {1, {2}, 1, 63, 1, 0},       //
     {1, {1}, 1, 63, 1, 0},       //
-    {1, {0}, 1, 63, 1, 0},       //
-};
-constexpr size_t kNumScans4 = ARRAY_SIZE(kScanScript4);
+    {1, {0}, 1, 63, 1, 0}        //
+});
+constexpr size_t kNumScans4 = kScanScript4.size();
 
-constexpr jpeg_scan_info kScanScript5[] = {
+constexpr auto kScanScript5 = to_array<jpeg_scan_info>({
     {3, {0, 1, 2}, 0, 0, 0, 1},  //
     {1, {0}, 1, 5, 0, 2},        //
     {1, {1}, 1, 5, 0, 2},        //
@@ -118,12 +117,12 @@ constexpr jpeg_scan_info kScanScript5[] = {
     {3, {0, 1, 2}, 0, 0, 1, 0},  //
     {1, {0}, 1, 63, 1, 0},       //
     {1, {1}, 1, 63, 1, 0},       //
-    {1, {2}, 1, 63, 1, 0},       //
-};
-constexpr size_t kNumScans5 = ARRAY_SIZE(kScanScript5);
+    {1, {2}, 1, 63, 1, 0}        //
+});
+constexpr size_t kNumScans5 = kScanScript5.size();
 
 // default progressive mode of jpegli
-constexpr jpeg_scan_info kScanScript6[] = {
+constexpr auto kScanScript6 = to_array<jpeg_scan_info>({
     {3, {0, 1, 2}, 0, 0, 0, 0},  //
     {1, {0}, 1, 2, 0, 0},        //
     {1, {1}, 1, 2, 0, 0},        //
@@ -137,8 +136,8 @@ constexpr jpeg_scan_info kScanScript6[] = {
     {1, {0}, 3, 63, 1, 0},       //
     {1, {1}, 3, 63, 1, 0},       //
     {1, {2}, 3, 63, 1, 0},       //
-};
-constexpr size_t kNumScans6 = ARRAY_SIZE(kScanScript6);
+});
+constexpr size_t kNumScans6 = kScanScript6.size();
 
 // Adapt RGB scan info to grayscale jpegs.
 void FilterScanComponents(const jpeg_compress_struct* cinfo,
@@ -163,12 +162,12 @@ Status SetJpegProgression(int progressive_id,
     jpeg_simple_progression(cinfo);
     return true;
   }
-  constexpr const jpeg_scan_info* kScanScripts[] = {kScanScript1, kScanScript2,
-                                                    kScanScript3, kScanScript4,
-                                                    kScanScript5, kScanScript6};
-  constexpr size_t kNumScans[] = {kNumScans1, kNumScans2, kNumScans3,
-                                  kNumScans4, kNumScans5, kNumScans6};
-  if (progressive_id > static_cast<int>(ARRAY_SIZE(kNumScans))) {
+  const jpeg_scan_info* kScanScripts[] = {
+      kScanScript1.data(), kScanScript2.data(), kScanScript3.data(),
+      kScanScript4.data(), kScanScript5.data(), kScanScript6.data()};
+  constexpr auto kNumScans = to_array<size_t>(
+      {kNumScans1, kNumScans2, kNumScans3, kNumScans4, kNumScans5, kNumScans6});
+  if (progressive_id > static_cast<int>(kNumScans.size())) {
     return JXL_FAILURE("Unknown jpeg scan script id %d", progressive_id);
   }
   const jpeg_scan_info* scan_script = kScanScripts[progressive_id - 1];
@@ -313,7 +312,7 @@ Status EncodeWithLibJpeg(const PackedImage& image, const JxlBasicInfo& info,
   if (cinfo.num_components == static_cast<int>(image.format.num_channels) &&
       image.format.data_type == JXL_TYPE_UINT8) {
     for (size_t y = 0; y < info.ysize; ++y) {
-      memcpy(&row_bytes[0], pixels + y * image.stride, image.stride);
+      memcpy(row_bytes.data(), pixels + y * image.stride, image.stride);
       JSAMPROW row[] = {row_bytes.data()};
       jpeg_write_scanlines(&cinfo, row, 1);
     }
@@ -539,7 +538,7 @@ class JPEGEncoder : public Encoder {
     return formats;
   }
   Status Encode(const PackedPixelFile& ppf, EncodedImage* encoded_image,
-                ThreadPool* pool = nullptr) const override {
+                ThreadPool* pool) const override {
     JXL_RETURN_IF_ERROR(VerifyBasicInfo(ppf.info));
     JpegEncoder jpeg_encoder = JpegEncoder::kLibJpeg;
     JpegParams params;
