@@ -10,6 +10,7 @@
 #include <vector>
 
 #include "lib/jxl/fields.h"
+#include "lib/jxl/image_ops.h"
 #include "lib/jxl/modular/modular_image.h"
 #include "lib/jxl/modular/options.h"
 
@@ -22,7 +23,7 @@ constexpr static int64_t kPredictionRound = ((1 << kPredExtraBits) >> 1) - 1;
 constexpr static size_t kNumProperties = 1;
 
 struct Header : public Fields {
-  const char *Name() const override { return "WeightedPredictorHeader"; }
+  JXL_FIELDS_NAME(WeightedPredictorHeader)
   // TODO(janwas): move to cc file, avoid including fields.h.
   Header() { Bundle::Init(this); }
 
@@ -62,13 +63,23 @@ struct State {
   pixel_type_w pred = 0;  // *before* removing the added bits.
   std::vector<uint32_t> pred_errors[kNumPredictors];
   std::vector<int32_t> error;
-  Header header;
+  const Header header;
 
   // Allows to approximate division by a number from 1 to 64.
-  uint32_t divlookup[64];
+  //  for (int i = 0; i < 64; i++) divlookup[i] = (1 << 24) / (i + 1);
+
+  const uint32_t divlookup[64] = {
+      16777216, 8388608, 5592405, 4194304, 3355443, 2796202, 2396745, 2097152,
+      1864135,  1677721, 1525201, 1398101, 1290555, 1198372, 1118481, 1048576,
+      986895,   932067,  883011,  838860,  798915,  762600,  729444,  699050,
+      671088,   645277,  621378,  599186,  578524,  559240,  541200,  524288,
+      508400,   493447,  479349,  466033,  453438,  441505,  430185,  419430,
+      409200,   399457,  390167,  381300,  372827,  364722,  356962,  349525,
+      342392,   335544,  328965,  322638,  316551,  310689,  305040,  299593,
+      294337,   289262,  284359,  279620,  275036,  270600,  266305,  262144};
 
   constexpr static pixel_type_w AddBits(pixel_type_w x) {
-    return uint64_t(x) << kPredExtraBits;
+    return static_cast<uint64_t>(x) << kPredExtraBits;
   }
 
   State(Header header, size_t xsize, size_t ysize) : header(header) {
@@ -78,10 +89,6 @@ struct State {
       pred_errors[i].resize((xsize + 2) * 2);
     }
     error.resize((xsize + 2) * 2);
-    // Initialize division lookup table.
-    for (int i = 0; i < 64; i++) {
-      divlookup[i] = (1 << 24) / (i + 1);
-    }
   }
 
   // Approximates 4+(maxweight<<24)/(x+1), avoiding division
@@ -282,15 +289,15 @@ struct FlatDecisionNode {
     PropertyVal splitval0;
     Predictor predictor;
   };
-  uint32_t childID;  // childID is ctx id if leaf.
   // Property+splitval of the two child nodes.
   union {
     PropertyVal splitvals[2];
     int32_t multiplier;
   };
+  uint32_t childID;  // childID is ctx id if leaf.
   union {
-    int32_t properties[2];
-    int64_t predictor_offset;
+    int16_t properties[2];
+    int32_t predictor_offset;
   };
 };
 using FlatTree = std::vector<FlatDecisionNode>;
@@ -301,21 +308,27 @@ class MATreeLookup {
   struct LookupResult {
     uint32_t context;
     Predictor predictor;
-    int64_t offset;
+    int32_t offset;
     int32_t multiplier;
   };
-  LookupResult Lookup(const Properties &properties) const {
+  JXL_INLINE LookupResult Lookup(const Properties &properties) const {
     uint32_t pos = 0;
     while (true) {
-      const FlatDecisionNode &node = nodes_[pos];
-      if (node.property0 < 0) {
-        return {node.childID, node.predictor, node.predictor_offset,
-                node.multiplier};
-      }
-      bool p0 = properties[node.property0] <= node.splitval0;
-      uint32_t off0 = properties[node.properties[0]] <= node.splitvals[0];
-      uint32_t off1 = 2 | (properties[node.properties[1]] <= node.splitvals[1]);
-      pos = node.childID + (p0 ? off1 : off0);
+#define TRAVERSE_THE_TREE                                                      \
+  {                                                                            \
+    const FlatDecisionNode &node = nodes_[pos];                                \
+    if (node.property0 < 0) {                                                  \
+      return {node.childID, node.predictor, node.predictor_offset,             \
+              node.multiplier};                                                \
+    }                                                                          \
+    bool p0 = properties[node.property0] <= node.splitval0;                    \
+    uint32_t off0 = properties[node.properties[0]] <= node.splitvals[0];       \
+    uint32_t off1 = 2 | (properties[node.properties[1]] <= node.splitvals[1]); \
+    pos = node.childID + (p0 ? off1 : off0);                                   \
+  }
+
+      TRAVERSE_THE_TREE;
+      TRAVERSE_THE_TREE;
     }
   }
 
@@ -398,46 +411,6 @@ struct PredictionResult {
   int32_t multiplier;
 };
 
-inline std::string PropertyName(size_t i) {
-  static_assert(kNumNonrefProperties == 16, "Update this function");
-  switch (i) {
-    case 0:
-      return "c";
-    case 1:
-      return "g";
-    case 2:
-      return "y";
-    case 3:
-      return "x";
-    case 4:
-      return "|N|";
-    case 5:
-      return "|W|";
-    case 6:
-      return "N";
-    case 7:
-      return "W";
-    case 8:
-      return "W-WW-NW+NWW";
-    case 9:
-      return "W+N-NW";
-    case 10:
-      return "W-NW";
-    case 11:
-      return "NW-N";
-    case 12:
-      return "N-NE";
-    case 13:
-      return "N-NN";
-    case 14:
-      return "W-WW";
-    case 15:
-      return "WGH";
-    default:
-      return "ch[" + ToString(15 - (int)i) + "]";
-  }
-}
-
 inline void InitPropsRow(
     Properties *p,
     const std::array<pixel_type, kNumStaticProperties> &static_props,
@@ -455,6 +428,7 @@ enum PredictorMode {
   kUseWP = 2,
   kForceComputeProperties = 4,
   kAllPredictions = 8,
+  kNoEdgeCases = 16
 };
 
 JXL_INLINE pixel_type_w PredictOne(Predictor p, pixel_type_w left,
@@ -500,7 +474,7 @@ JXL_INLINE pixel_type_w PredictOne(Predictor p, pixel_type_w left,
 }
 
 template <int mode>
-inline PredictionResult Predict(
+JXL_INLINE PredictionResult Predict(
     Properties *p, size_t w, const pixel_type *JXL_RESTRICT pp,
     const intptr_t onerow, const size_t x, const size_t y, Predictor predictor,
     const MATreeLookup *lookup, const Channel *references,
@@ -509,20 +483,22 @@ inline PredictionResult Predict(
   size_t offset = 3;
   constexpr bool compute_properties =
       mode & kUseTree || mode & kForceComputeProperties;
-  pixel_type_w left = (x ? pp[-1] : (y ? pp[-onerow] : 0));
-  pixel_type_w top = (y ? pp[-onerow] : left);
-  pixel_type_w topleft = (x && y ? pp[-1 - onerow] : left);
-  pixel_type_w topright = (x + 1 < w && y ? pp[1 - onerow] : top);
-  pixel_type_w leftleft = (x > 1 ? pp[-2] : left);
-  pixel_type_w toptop = (y > 1 ? pp[-onerow - onerow] : top);
-  pixel_type_w toprightright = (x + 2 < w && y ? pp[2 - onerow] : topright);
+  constexpr bool nec = mode & kNoEdgeCases;
+  pixel_type_w left = (nec || x ? pp[-1] : (y ? pp[-onerow] : 0));
+  pixel_type_w top = (nec || y ? pp[-onerow] : left);
+  pixel_type_w topleft = (nec || (x && y) ? pp[-1 - onerow] : left);
+  pixel_type_w topright = (nec || (x + 1 < w && y) ? pp[1 - onerow] : top);
+  pixel_type_w leftleft = (nec || x > 1 ? pp[-2] : left);
+  pixel_type_w toptop = (nec || y > 1 ? pp[-onerow - onerow] : top);
+  pixel_type_w toprightright =
+      (nec || (x + 2 < w && y) ? pp[2 - onerow] : topright);
 
   if (compute_properties) {
     // location
     (*p)[offset++] = x;
     // neighbors
-    (*p)[offset++] = std::abs(top);
-    (*p)[offset++] = std::abs(left);
+    (*p)[offset++] = top > 0 ? top : -top;
+    (*p)[offset++] = left > 0 ? left : -left;
     (*p)[offset++] = top;
     (*p)[offset++] = left;
 
@@ -545,7 +521,7 @@ inline PredictionResult Predict(
     wp_pred = wp_state->Predict<compute_properties>(
         x, y, w, top, left, topright, topleft, toptop, p, offset);
   }
-  if (compute_properties) {
+  if (!nec && compute_properties) {
     offset += weighted::kNumProperties;
     // Extra properties.
     const pixel_type *JXL_RESTRICT rp = references->Row(x);
@@ -563,8 +539,9 @@ inline PredictionResult Predict(
   }
   if (mode & kAllPredictions) {
     for (size_t i = 0; i < kNumModularPredictors; i++) {
-      predictions[i] = PredictOne((Predictor)i, left, top, toptop, topleft,
-                                  topright, leftleft, toprightright, wp_pred);
+      predictions[i] =
+          PredictOne(static_cast<Predictor>(i), left, top, toptop, topleft,
+                     topright, leftleft, toprightright, wp_pred);
     }
   }
   result.guess += PredictOne(predictor, left, top, toptop, topleft, topright,
@@ -604,6 +581,15 @@ inline PredictionResult PredictTreeNoWP(Properties *p, size_t w,
       p, w, pp, onerow, x, y, Predictor::Zero, &tree_lookup, &references,
       /*wp_state=*/nullptr, /*predictions=*/nullptr);
 }
+// Only use for y > 1, x > 1, x < w-2, and empty references
+JXL_INLINE PredictionResult
+PredictTreeNoWPNEC(Properties *p, size_t w, const pixel_type *JXL_RESTRICT pp,
+                   const intptr_t onerow, const int x, const int y,
+                   const MATreeLookup &tree_lookup, const Channel &references) {
+  return detail::Predict<detail::kUseTree | detail::kNoEdgeCases>(
+      p, w, pp, onerow, x, y, Predictor::Zero, &tree_lookup, &references,
+      /*wp_state=*/nullptr, /*predictions=*/nullptr);
+}
 
 inline PredictionResult PredictTreeWP(Properties *p, size_t w,
                                       const pixel_type *JXL_RESTRICT pp,
@@ -616,6 +602,18 @@ inline PredictionResult PredictTreeWP(Properties *p, size_t w,
       p, w, pp, onerow, x, y, Predictor::Zero, &tree_lookup, &references,
       wp_state, /*predictions=*/nullptr);
 }
+JXL_INLINE PredictionResult PredictTreeWPNEC(Properties *p, size_t w,
+                                             const pixel_type *JXL_RESTRICT pp,
+                                             const intptr_t onerow, const int x,
+                                             const int y,
+                                             const MATreeLookup &tree_lookup,
+                                             const Channel &references,
+                                             weighted::State *wp_state) {
+  return detail::Predict<detail::kUseTree | detail::kUseWP |
+                         detail::kNoEdgeCases>(
+      p, w, pp, onerow, x, y, Predictor::Zero, &tree_lookup, &references,
+      wp_state, /*predictions=*/nullptr);
+}
 
 inline PredictionResult PredictLearn(Properties *p, size_t w,
                                      const pixel_type *JXL_RESTRICT pp,
@@ -639,6 +637,29 @@ inline void PredictLearnAll(Properties *p, size_t w,
       p, w, pp, onerow, x, y, Predictor::Zero,
       /*lookup=*/nullptr, &references, wp_state, predictions);
 }
+inline PredictionResult PredictLearnNEC(Properties *p, size_t w,
+                                        const pixel_type *JXL_RESTRICT pp,
+                                        const intptr_t onerow, const int x,
+                                        const int y, Predictor predictor,
+                                        const Channel &references,
+                                        weighted::State *wp_state) {
+  return detail::Predict<detail::kForceComputeProperties | detail::kUseWP |
+                         detail::kNoEdgeCases>(
+      p, w, pp, onerow, x, y, predictor, /*lookup=*/nullptr, &references,
+      wp_state, /*predictions=*/nullptr);
+}
+
+inline void PredictLearnAllNEC(Properties *p, size_t w,
+                               const pixel_type *JXL_RESTRICT pp,
+                               const intptr_t onerow, const int x, const int y,
+                               const Channel &references,
+                               weighted::State *wp_state,
+                               pixel_type_w *predictions) {
+  detail::Predict<detail::kForceComputeProperties | detail::kUseWP |
+                  detail::kAllPredictions | detail::kNoEdgeCases>(
+      p, w, pp, onerow, x, y, Predictor::Zero,
+      /*lookup=*/nullptr, &references, wp_state, predictions);
+}
 
 inline void PredictAllNoWP(size_t w, const pixel_type *JXL_RESTRICT pp,
                            const intptr_t onerow, const int x, const int y,
