@@ -281,9 +281,9 @@ static std::string PrintFormattedEntries(
     }
     // All except the first one are right-aligned, the first one is the name,
     // others are numbers with digits matching from the right.
-    if (i == 0) out += value.c_str();
+    if (i == 0) out += value;
     out += std::string(numspaces, ' ');
-    if (i != 0) out += value.c_str();
+    if (i != 0) out += value;
   }
   return out + "\n";
 }
@@ -303,9 +303,9 @@ std::string PrintHeader(const std::vector<std::string>& extra_metrics_names) {
     const std::string& label = descriptors[i].label;
     int numspaces = descriptors[i].width - label.size();
     // All except the first one are right-aligned.
-    if (i == 0) out += label.c_str();
+    if (i == 0) out += label;
     out += std::string(numspaces, ' ');
-    if (i != 0) out += label.c_str();
+    if (i != 0) out += label;
   }
   for (const std::string& em : extra_metrics_names) {
     int numspaces = ExtraMetricDescriptor().width - em.size();
